#!/usr/bin/env python

import markdown
from markdown.util import etree


class VideoExtension(markdown.Extension):
    def __init__(self, **kwargs):
        self.config = {
            "dailymotion_width": ["480", "Width for Dailymotion videos"],
            "dailymotion_height": ["270", "Height for Dailymotion videos"],
            "metacafe_width": ["440", "Width for Metacafe videos"],
            "metacafe_height": ["248", "Height for Metacafe videos"],
            "veoh_width": ["410", "Width for Veoh videos"],
            "veoh_height": ["341", "Height for Veoh videos"],
            "vimeo_width": ["500", "Width for Vimeo videos"],
            "vimeo_height": ["281", "Height for Vimeo videos"],
            "yahoo_width": ["624", "Width for Yahoo! videos"],
            "yahoo_height": ["351", "Height for Yahoo! videos"],
            "youtube_width": ["560", "Width for Youtube videos"],
            "youtube_height": ["315", "Height for Youtube videos"],
            "youtube_nocookie": [
                False,
                "Use youtube-nocookie.com instead of youtube.com",
            ],
        }

        # Override defaults with user settings
        for key, value in kwargs.items():
            self.setConfig(key, str(value))

    def add_inline(self, md, name, klass, re):
        pattern = klass(re)
        pattern.md = md
        pattern.ext = self
        md.inlinePatterns.add(name, pattern, "<reference")

    def extendMarkdown(self, md, md_globals):
        self.add_inline(
            md,
            "dailymotion",
            Dailymotion,
            r"([^(]|^)https?://www\.dailymotion\.com/video/(?P<dailymotionid>[a-zA-Z0-9]+)(_[\w\-]*)?",  # noqa: E501
        )
        self.add_inline(
            md,
            "metacafe",
            Metacafe,
            r"([^(]|^)http://www\.metacafe\.com/watch/(?P<metacafeid>\d+)/?(:?.+/?)",  # noqa: E501
        )
        self.add_inline(
            md,
            "veoh",
            Veoh,
            r"([^(]|^)http://www\.veoh\.com/\S*(#watch%3D|watch/)(?P<veohid>\w+)",  # noqa: E501
        )
        self.add_inline(
            md, "vimeo", Vimeo, r"([^(]|^)http://(www.|)vimeo\.com/(?P<vimeoid>\d+)\S*"  # noqa: E501
        )
        self.add_inline(md, "yahoo", Yahoo, r"([^(]|^)http://screen\.yahoo\.com/.+/?")  # noqa: E501
        self.add_inline(
            md,
            "youtube",
            Youtube,
            r"([^(]|^)https?://www\.youtube\.com/watch\?\S*v=(?P<youtubeid>\S[^&/]+)",  # noqa: E501
        )
        self.add_inline(
            md,
            "youtube_short",
            Youtube,
            r"([^(]|^)https?://youtu\.be/(?P<youtubeid>\S[^?&/]+)?",
        )


class Dailymotion(markdown.inlinepatterns.Pattern):
    def handleMatch(self, m):
        url = "//www.dailymotion.com/embed/video/%s" % m.group("dailymotionid")
        width = self.ext.config["dailymotion_width"][0]
        height = self.ext.config["dailymotion_height"][0]
        return render_iframe(url, width, height)


class Metacafe(markdown.inlinepatterns.Pattern):
    def handleMatch(self, m):
        url = "//www.metacafe.com/embed/%s/" % m.group("metacafeid")
        width = self.ext.config["metacafe_width"][0]
        height = self.ext.config["metacafe_height"][0]
        return render_iframe(url, width, height)


class Veoh(markdown.inlinepatterns.Pattern):
    def handleMatch(self, m):
        url = "//www.veoh.com/videodetails2.swf?permalinkId=%s" % m.group("veohid")  # noqa: E501
        width = self.ext.config["veoh_width"][0]
        height = self.ext.config["veoh_height"][0]
        return flash_object(url, width, height)


class Vimeo(markdown.inlinepatterns.Pattern):
    def handleMatch(self, m):
        url = "//player.vimeo.com/video/%s" % m.group("vimeoid")
        width = self.ext.config["vimeo_width"][0]
        height = self.ext.config["vimeo_height"][0]
        return render_iframe(url, width, height)


class Yahoo(markdown.inlinepatterns.Pattern):
    def handleMatch(self, m):
        url = m.string + "?format=embed&player_autoplay=false"
        width = self.ext.config["yahoo_width"][0]
        height = self.ext.config["yahoo_height"][0]
        return render_iframe(url, width, height)


class Youtube(markdown.inlinepatterns.Pattern):
    def handleMatch(self, m):
        if self.ext.config["youtube_nocookie"][0]:
            url = "//www.youtube-nocookie.com/embed/%s" % m.group("youtubeid")
        else:
            url = "//www.youtube.com/embed/%s" % m.group("youtubeid")
        width = self.ext.config["youtube_width"][0]
        height = self.ext.config["youtube_height"][0]
        return render_iframe(url, width, height)


def render_iframe(url, width, height):
    iframe = etree.Element("iframe")
    iframe.set("width", width)
    iframe.set("height", height)
    iframe.set("src", url)
    iframe.set("allowfullscreen", "true")
    iframe.set("frameborder", "0")
    return iframe


def flash_object(url, width, height):
    obj = etree.Element("object")
    obj.set("type", "application/x-shockwave-flash")
    obj.set("width", width)
    obj.set("height", height)
    obj.set("data", url)
    param = etree.Element("param")
    param.set("name", "movie")
    param.set("value", url)
    obj.append(param)
    param = etree.Element("param")
    param.set("name", "allowFullScreen")
    param.set("value", "true")
    obj.append(param)
    return obj


def makeExtension(**kwargs):
    return VideoExtension(**kwargs)


if __name__ == "__main__":
    import doctest

    doctest.testmod()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import json
import base64
import requests
from .settings import MARTOR_IMGUR_CLIENT_ID, MARTOR_IMGUR_API_KEY

requests.packages.urllib3.disable_warnings()


def imgur_uploader(image):
    """
    Basic imgur uploader return as json data.
    :param `image` is from `request.FILES['markdown-image-upload']`
    :return json response
    """
    api_url = "https://api.imgur.com/3/upload.json"
    headers = {"Authorization": "Client-ID " + MARTOR_IMGUR_CLIENT_ID}
    response = requests.post(
        api_url,
        headers=headers,
        data={
            "key": MARTOR_IMGUR_API_KEY,
            "image": base64.b64encode(image.read()),
            "type": "base64",
            "name": image.name,
        },
    )

    if response.status_code == 200:
        response_data = json.loads(response.content.decode("utf-8"))
        return json.dumps(
            {
                "status": response_data["status"],
                "link": response_data["data"]["link"],
                "name": response_data["data"]["name"],
            }
        )

    elif response.status_code == 415:
        # Unsupport File type
        return json.dumps(
            {
                "status": response.status_code,
                "error": response.reason,
            }
        )

    return json.dumps(
        {
            "status": response.status_code,
            "error": response.content.decode("utf-8"),
        }
    )



from django.views.generic.edit import CreateView

from .models import Post


class TestFormView(CreateView):
    template_name = "test_form_view.html"
    model = Post
    fields = ["description", "wiki"]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import django
from .views import (
    markdownfy_view,
    markdown_imgur_uploader,
    markdown_search_user,
)


if django.VERSION >= (2, 0):
    from django.urls import path

    urlpatterns = [
        path("markdownify/", markdownfy_view, name="martor_markdownfy"),
        path("uploader/", markdown_imgur_uploader, name="imgur_uploader"),
        path("search-user/", markdown_search_user, name="search_user_json"),
    ]
else:
    from django.conf.urls import url

    urlpatterns = [
        url(r"^markdownify/$", markdownfy_view, name="martor_markdownfy"),
        url(r"^uploader/$", markdown_imgur_uploader, name="imgur_uploader"),
        url(r"^search-user/$", markdown_search_user, name="search_user_json"),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib import admin

from .widgets import AdminMartorWidget
from .models import MartorField


class MartorModelAdmin(admin.ModelAdmin):

    formfield_overrides = {
        MartorField: {"widget": AdminMartorWidget},
    }



from django import template
from django.utils.safestring import mark_safe

from ..utils import markdownify

register = template.Library()


@register.filter
def safe_markdown(markdown_text):
    """
    Safe the markdown text as html ouput.

    Usage:
        {% load martortags %}
        {{ markdown_text|safe_markdown }}

    Example:
        {{ post.description|safe_markdown }}
    """
    return mark_safe(markdownify(markdown_text))



import re

from django.utils.html import strip_tags
from django.utils.functional import Promise
from django.core.serializers.json import DjangoJSONEncoder

try:
    from django.utils.encoding import force_str  # noqa: Django>=4.x
except ImportError:
    from django.utils.encoding import force_text as force_str  # noqa: Django<=3.x

import markdown
from .settings import (
    MARTOR_MARKDOWN_EXTENSIONS,
    MARTOR_MARKDOWN_EXTENSION_CONFIGS,
    ALLOWED_URL_SCHEMES,
)


def markdownify(markdown_text):
    """
    Render the markdown content to HTML.

    Basic:
        >>> from martor.utils import markdownify
        >>> content = "![awesome](http://i.imgur.com/hvguiSn.jpg)"
        >>> markdownify(content)
        '<p><img alt="awesome" src="http://i.imgur.com/hvguiSn.jpg" /></p>'
        >>>
    """
    # Strip HTML tags
    markdown_text = strip_tags(markdown_text)

    # Sanitize Markdown links
    # https://github.com/netbox-community/netbox/commit/5af2b3c2f577a01d177cb24cda1019551a2a4b64
    schemes = "|".join(ALLOWED_URL_SCHEMES)
    pattern = fr"\[(.+)\]\((?!({schemes})).*:(.+)\)"
    markdown_text = re.sub(
        pattern,
        "[\\1](\\3)",
        markdown_text,
        flags=re.IGNORECASE,
    )

    return markdown.markdown(
        markdown_text,
        extensions=MARTOR_MARKDOWN_EXTENSIONS,
        extension_configs=MARTOR_MARKDOWN_EXTENSION_CONFIGS,
    )


class LazyEncoder(DjangoJSONEncoder):
    """
    This problem because we found error encoding,
    as docs says, django has special `DjangoJSONEncoder` at
    https://docs.djangoproject.com/en/dev/topics/serialization/#serialization-formats-json
    also discused in this answer: http://stackoverflow.com/a/31746279/6396981

    Usage:
        >>> data = {}
        >>> json.dumps(data, cls=LazyEncoder)
    """

    def default(self, obj):
        if isinstance(obj, Promise):
            return force_str(obj)
        return super(LazyEncoder, self).default(obj)



import markdown


class EscapeHtml(markdown.Extension):
    def extendMarkdown(self, md):
        md.preprocessors.deregister("html_block")
        md.inlinePatterns.deregister("html")


def makeExtension(*args, **kwargs):
    return EscapeHtml(*args, **kwargs)


if __name__ == "__main__":
    import doctest

    doctest.testmod()
